// Copyright IBM Corp. 2016. All Rights Reserved.
// Node module: apiconnect-cli-util
// US Government Users Restricted Rights - Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.

/********************************************************* {COPYRIGHT-TOP} ***
 * IBM Confidential
 * OCO Source Materials
 * 5725-C11, 5725-C12, 7198-8FX, 5725-F70, 5725-F71, 5725-F72, 5725-L30
 *
 * (C) Copyright IBM Corporation 2015  All Rights Reserved.
 *
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 ********************************************************* {COPYRIGHT-END} **/
/*eslint brace-style: 0, max-len: 0*/
var async = require('async');
var u = require('../util/util');
var debug = require('debug')('apiml:auth');
var ClientRequest = require('../util/apim-request');
var Netrc = require('../util/netrc');

var _apimanager = 'apimanager';


/**
 * Authenticate user into APIC management defined by <tt>host</tt> using credentials defined by <tt>username/password</tt>
 *
 * @param username
 * @param password
 * @param host
 * @param cb {Function(err, rsp, body} callback function that is invoked once authentication is completed
 */
function authenticate(username, password, host, cb) {
  var opts = u.requestOptions(username, password, host);
  opts.url += '/v1/me';
  ClientRequest.get(opts, cb);
};
/**
 * Update the global configuration properties once user has logged in.
 *
 * @param username
 * @param password
 * @param host g
 */
function saveCredentials(username, password, host) {
  Netrc.setCredential(host, username, password);
};

module.exports.login = login;
/**
 * Login into an APIM management stack.
 *
 * @param username
 * @param password
 * @param host
 * @param callback {Function(err)} if no error is return then err is null - and that's mean that the login has
 *         been successfull.
 */
function login(username, password, host, callback) {
  var tasks = [];
  if (username.indexOf(_apimanager + '/') < 0) {
    username = _apimanager + '/' + username;
  }
  tasks.push(function(cb) {
    cb(null, username, password, host);
  });

  tasks.push(function(username, password, host, cb) {
    authenticate(username, password, host, function(err, rsp, body) {
      if (err) {
        cb(err);
      }
      else if (rsp.statusCode === 200) {
        body = typeof body === 'string' ? JSON.parse(body) : body;
        cb(null, username, password, host);
      }
      else if (rsp.statusCode === 401) {
        var error = new Error('Invalid credentials.', 401);
        cb(error);
      }
      else {
        err = new Error('Unexpected Result - status code: ' + rsp.statusCode + ' error: ' + body, rsp.statusCode);
        cb(err);
      }
    });
  });

  tasks.push(function(username, password, host, cb) {
    saveCredentials(username, password, host);
    cb(null);
  });

  tasks.push(function(cb) {
    debug('Done with authentication and validation.');
    cb(null);
  });

  async.waterfall(tasks, function(err) {
    if (err) {
      debug('Error' + err.toString());
      callback(err);

    }
    else {
      callback(null);
    }
  });

};

module.exports.logout = logout;
function logout(server, callback) {
  Netrc.deleteCredential(server);
  callback(null);
};
