// Copyright IBM Corp. 2016. All Rights Reserved.
// Node module: apiconnect-cli-util
// US Government Users Restricted Rights - Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.

/********************************************************* {COPYRIGHT-TOP} ***
 * IBM Confidential
 * OCO Source Materials
 * 5725-C11, 5725-C12, 7198-8FX, 5725-F70, 5725-F71, 5725-F72, 5725-L30
 *
 * (C) Copyright IBM Corporation 2015  All Rights Reserved.
 *
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 ********************************************************* {COPYRIGHT-END} **/
/*eslint brace-style: 0, max-len: 0*/
var util = require('util');
var u = require('./util');
/**
 * Marshall the data from one format to another.
 */

function Marshaller() {}
/**
 * Should be invoked before a command gets invoked
 */
Marshaller.prototype.pre = function() {};
/**
 * Should be invoked after a command have been executed
 */
Marshaller.prototype.post = function() {};
/**
 * Marshall data and return it as an object.
 *
 * @param data {object}
 */
Marshaller.prototype.marshall = function(data) {
  return data;
};


Marshaller.prototype.error = function(error) {
  console.error(u.stringifyError(error));
  process.exit(1);
};
/**
 * Return the name of the marshaller
 *
 * @returns {string}
 */
Marshaller.prototype.name = function() {
  return 'no-op';
};


var marshaller = {};

var register = function(marshaller) {
  marshaller[marshaller.name()] = marshaller;
};

var lookup = function(name) {
  return marshaller[name];
};

//register internal formatter
var noOpMarshaller = new Marshaller();
register(noOpMarshaller);

module.exports = Marshaller;
module.exports.register = register;
module.exports.lookup = lookup;

function PrettyPrintJsonMarshaller() {
}

util.inherits(PrettyPrintJsonMarshaller, Marshaller);


PrettyPrintJsonMarshaller.prototype.format = function(data) {
  return JSON.stringify(data, null, '  ');
};


PrettyPrintJsonMarshaller.prototype.name = function() {
  return 'pretty-print-json';
};
